package items;

import java.util.ArrayList;
import java.util.List;

/**
 * This class represents a general item which can be stored in an inventory
 */
public abstract class Item {
    final String name;
    final String description;
    static final List<Item> availableItems = new ArrayList<>();

    /**
     * Instantiates a new item
     * @param name <p>The name of the item</p>
     * @param description <p>The description of the item</p>
     */
    public Item(String name, String description) {
        this.name = name;
        this.description = description;
    }

    /**
     * Returns an item by the given name
     * @param name <p>The name of the item to get</p>
     * @return <p>An item with the name, or null if no such item exists</p>
     */
    public static Item getItemByName(String name) {
        if (availableItems.isEmpty()) {
            initializeItems();
        }
        for (Item item : availableItems) {
            if (item.name.equals(name)) {
                return item;
            }
        }
        return null;
    }

    /**
     * Initializes all known items so that they are usable
     */
    private static void initializeItems() {
        availableItems.add(new PokeBall());
        availableItems.add(new GreatBall());
        availableItems.add(new HyperPotion());
        availableItems.add(new MasterBall());
        availableItems.add(new MaxPotion());
        availableItems.add(new Potion());
        availableItems.add(new Revive());
        availableItems.add(new SuperPotion());
        availableItems.add(new UltraBall());
    }

    /**
     * Makes a nice string describing the item
     * @return <p>Name: Description</p>
     */
    public String toString() {
        return String.format("%s: %s", this.name, this.description);
    }

    /**
     * Returns a string used for saving the item
     * @return <p>Name;Description</p>
     */
    public String saveString() {
        return String.format("%s", this.name);
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (!Item.class.isAssignableFrom(obj.getClass())) {
            return false;
        }
        return this.name.equals(((Item) obj).name);
    }
}
